import React, { cloneElement, isValidElement, useEffect, useMemo, useRef, useState } from "react";
import { addToQueue, removeFromQueue, speakFromQueue, subscribe, dequeue, clearQueueHook, clearQueueUnload, clearQueue } from "./queue.js";
import { ArrayToText, JSXToArray, cancel, findCharIndex, getIndex, isParent, sanitize } from "./utils.js";
export function useQueue() {
    const [queue, setQueue] = useState([]);
    useEffect(() => subscribe((queue) => setQueue([...queue])), []);
    return { queue, dequeue, clearQueue: clearQueueHook };
}
export function useSpeech({ text, pitch = 1, rate = 1, volume = 1, lang, voiceURI, highlightText = false, highlightProps = { style: { backgroundColor: "yellow" } }, preserveUtteranceQueue = false, onError = console.error, onStart, onResume, onPause, onStop, onBoundary, onQueueChange, }) {
    const [speechStatus, speechStatusRef, setSpeechStatus] = useStateRef("stopped");
    const [speakingWord, setSpeakingWord] = useState();
    const utteranceRef = useRef();
    const [words, stringifiedWords] = useMemo(() => {
        const words = JSXToArray(text);
        return [words, JSON.stringify(words)];
    }, [text]);
    function start() {
        const synth = window.speechSynthesis;
        if (!synth)
            return onError(new Error("Browser not supported! Try some other browser."));
        if (speechStatus === "paused")
            return synth.resume();
        if (speechStatus === "queued")
            return;
        const utterance = new SpeechSynthesisUtterance(sanitize(ArrayToText(words)));
        utterance.pitch = pitch;
        utterance.rate = rate;
        utterance.volume = volume;
        if (lang)
            utterance.lang = lang;
        if (voiceURI) {
            if (!Array.isArray(voiceURI))
                voiceURI = [voiceURI];
            const voices = synth.getVoices();
            for (let i = 0; i < voiceURI.length; i++) {
                const uri = voiceURI[i];
                const voice = voices.find((voice) => voice.voiceURI === uri);
                if (voice) {
                    utterance.voice = voice;
                    break;
                }
            }
        }
        const stopEventHandler = (event) => {
            window.removeEventListener("beforeunload", clearQueueUnload);
            setSpeechStatus("stopped");
            setSpeakingWord(null);
            utterance.onstart = null;
            utterance.onresume = null;
            utterance.onpause = null;
            utterance.onend = null;
            utterance.onerror = null;
            utterance.onboundary = null;
            removeFromQueue(utterance, onQueueChange);
            speakFromQueue();
            onStop === null || onStop === void 0 ? void 0 : onStop(event);
        };
        utterance.onstart = (event) => {
            window.addEventListener("beforeunload", clearQueueUnload);
            setSpeechStatus("started");
            onStart === null || onStart === void 0 ? void 0 : onStart(event);
        };
        utterance.onresume = (event) => {
            setSpeechStatus("started");
            onResume === null || onResume === void 0 ? void 0 : onResume(event);
        };
        utterance.onpause = (event) => {
            setSpeechStatus("paused");
            onPause === null || onPause === void 0 ? void 0 : onPause(event);
        };
        utterance.onend = stopEventHandler;
        utterance.onerror = stopEventHandler;
        utterance.onboundary = (event) => {
            if (highlightText)
                setSpeakingWord({ index: findCharIndex(words, event.charIndex), length: event.charLength });
            onBoundary === null || onBoundary === void 0 ? void 0 : onBoundary(event);
        };
        if (!preserveUtteranceQueue)
            clearQueue();
        addToQueue({ utterance, setSpeechStatus }, onQueueChange);
        if (synth.speaking) {
            if (preserveUtteranceQueue && speechStatus !== "started") {
                utteranceRef.current = utterance;
                return setSpeechStatus("queued");
            }
            else
                cancel();
        }
        else
            speakFromQueue();
        setSpeechStatus("started");
    }
    function pause() {
        var _a;
        if (speechStatus === "started")
            return (_a = window.speechSynthesis) === null || _a === void 0 ? void 0 : _a.pause();
        if (speechStatus === "queued")
            stop();
    }
    function stop(status = speechStatus) {
        if (status === "stopped")
            return;
        if (status !== "queued")
            return cancel();
        removeFromQueue(utteranceRef.current, onQueueChange);
        setSpeechStatus("stopped");
    }
    function highlightedText(element, parentIndex = "") {
        var _a;
        if (!isParent(parentIndex, speakingWord === null || speakingWord === void 0 ? void 0 : speakingWord.index))
            return element;
        if (Array.isArray(element))
            return element.map((child, index) => highlightedText(child, getIndex(parentIndex, index)));
        if (isValidElement(element))
            return cloneElement(element, { key: (_a = element.key) !== null && _a !== void 0 ? _a : Math.random() }, highlightedText(element.props.children, parentIndex));
        if (typeof element === "string" || typeof element === "number") {
            element = String(element);
            const { index, length } = speakingWord;
            const before = element.slice(0, +index.split("-").at(-1)).length;
            return (React.createElement("span", { key: index },
                element.slice(0, before),
                React.createElement("span", Object.assign({}, highlightProps), element.slice(before, before + length)),
                element.slice(before + length)));
        }
        return element;
    }
    useEffect(() => {
        return () => stop(speechStatusRef.current);
    }, [stringifiedWords]);
    return {
        Text: () => highlightedText(text),
        speechStatus,
        isInQueue: speechStatus === "started" || speechStatus === "queued",
        start,
        pause,
        stop: () => stop(),
    };
}
function useStateRef(init) {
    const [state, setState] = useState(init);
    const ref = useRef(init);
    function setStateRef(value) {
        ref.current = value;
        setState(value);
    }
    return [state, ref, setStateRef];
}
